"""
author : MaHaoYang
publish date : 2019-12-21
name : link rank
expression : Rank = ∑ (ln((ln(TextLength+1) + TagMatrix * WeightMatrix) * α)) ± δ;  α = 2 ; δ ≈ 5 * 10e-3

"""

import re
import copy
import math
import traceback

import cchardet
import lxml
import lxml.html
from lxml.html import HtmlComment
from lxml import etree

# from .format import Format
from spider_tool.format import Format

REGEXES = {
    'okMaybeItsACandidateRe': re.compile(
        'and|article|artical|body|column|main|shadow', re.I),
    'positiveRe': re.compile(
        ('article|arti|body|content|entry|hentry|main|page|'
         'artical|zoom|arti|context|message|editor|'
         'pagination|post|txt|text|blog|story'), re.I),
    'negativeRe': re.compile(
        ('copyright|combx|comment|com-|contact|foot|footer|footnote|decl|copy|'
         'notice|'
         'masthead|media|meta|outbrain|promo|related|scroll|link|pagebottom|bottom|'
         'other|shoutbox|sidebar|sponsor|shopping|tags|tool|widget'), re.I),
}

REGEXES2 = {
    'okMaybeItsACandidateRe': re.compile(
        'and|article|artical|body|column|main|shadow', re.I),
    'positiveRe': re.compile(
        ('article|arti|body|content|entry|hentry|main|page|'
         'artical|zoom|arti|context|message|editor|'
         'pagination|post|txt|text|blog|story|link|title|tit|time|pub_time|date'), re.I),
    'negativeRe': re.compile(
        ('copyright|combx|comment|com-|contact|foot|footer|footnote|decl|copy|'
         'notice|'
         'masthead|media|meta|outbrain|promo|related|scroll|pagebottom|bottom|'
         'other|shoutbox|sidebar|sponsor|shopping|tags|tool|widget'), re.I),
}


class TitleContent:
    def __init__(self, ):
        self.non_content_tag = set([
            'head',
            'meta',
            'script',
            'style',
            'object',
            'embed',
            'iframe',
            'marquee',
            'select',
        ])
        self.title = ''
        self.p_space = re.compile(r'\s')
        self.p_html = re.compile(r'<html|</html>', re.IGNORECASE | re.DOTALL)
        self.p_content_stop = re.compile(r'正文.*结束|正文下|相关阅读|声明')
        self.p_clean_tree = re.compile(r'author|post-add|copyright')
        self.find_zh = re.compile(u'[\u4e00-\u9fa5]').findall
        self.find_zh_en = re.compile(u'[a-zA-Z\d_\u4e00-\u9fa5]').findall
        self.find_date = re.compile(u'.*?年.*?月.*?日|发布时间|\d{4}\-\d{1,2}\-\d{1,2}|昨天|前天|今天|小时前|\d{1,2}:\d{2}').findall

    def get_title(self, doc):
        title = ''
        title_el = doc.xpath('//title')
        if title_el:
            title = title_el[0].text_content().strip()
        if len(title) < 7:
            tt = doc.xpath('//meta[@name="title"]')
            if tt:
                title = tt[0].get('content', '')
        if len(title) < 7:
            tt = doc.xpath('//*[contains(@id, "title") or contains(@class, "title")]')
            if not tt:
                tt = doc.xpath('//*[contains(@id, "font01") or contains(@class, "font01")]')
            if not tt:
                tt = doc.xpath('//*[contains(@id, "tit") or contains(@class, "tit")]')
            for t in tt:
                ti = t.text_content().strip()
                if ti in title and len(ti) * 2 > len(title):
                    title = ti
                    break
                if len(ti) > 20: continue
                if len(ti) > len(title) or len(ti) > 7:
                    title = ti
        return title

    def shorten_title(self, title):
        spliters = [' - ', '–', '—', '-', '|', '::']
        for s in spliters:
            if s not in title:
                continue
            tts = title.split(s)
            if len(tts) < 2:
                continue
            title = tts[0]
            break
        return title

    def calc_node_weight(self, node):
        weight = 1
        attr = '%s %s %s' % (
            node.get('class', ''),
            node.get('id', ''),
            node.get('style', '')
        )
        if attr:
            mm = REGEXES['negativeRe'].findall(attr)
            weight -= 2 * len(mm)
            mm = REGEXES['positiveRe'].findall(attr)
            weight += 4 * len(mm)
        if node.tag in ['div', 'p', 'table']:
            weight += 2
        return weight

    def calc_node_weight_links(self, node):
        weight = 1
        attr = '%s %s %s' % (
            node.get('class', ''),
            node.get('id', ''),
            node.get('style', '')
        )
        # if attr:
        #     mm = REGEXES2['negativeRe'].findall(attr)
        #     weight -= 2 * len(mm)
        #     mm = REGEXES2['positiveRe'].findall(attr)
        #     weight += 2 * len(mm)
        if node.tag in ['div', 'ol']:
            weight += 3
        if node.tag in ['ul']:
            weight += 4
        return weight

    c = 0
    date_count = 0

    def count_lines(self, node):
        self.c += 1
        # d = self.find_date(etree.tostring(node, encoding='utf-8').decode('utf-8'))
        # d = Format().time_normalization(etree.tostring(node, encoding='utf-8').decode('utf-8'))
        # if d:
        #     self.date_count += 1
        for ii in node:
            if ii.tag:
                self.count_lines(ii)

    weight = 0

    def calc_links(self, node, r=1):
        for i in node:
            if not i:
                continue
            if isinstance(i, HtmlComment):
                continue
            if i.tag in self.non_content_tag:
                continue
            attr = '%s%s%s' % (i.get('class', ''),
                               i.get('id', ''),
                               i.get('style'))
            if 'display' in attr and 'none' in attr:
                continue

            if i.tag:
                if i.tag in ['body']:
                    return 0
                ii = copy.deepcopy(i)
                for iii in range(5):
                    if not ii:
                        continue
                    ii = ii.getparent()
                    if not ii:
                        continue
                    if ii.tag in ['body']:
                        self.weight = self.weight / (8 - iii)
                if i.tag in ['li', 'a']:
                    self.weight += 4
                ii = copy.deepcopy(i)
                for iii in range(2):
                    if not ii:
                        continue
                    ii = ii.getparent()
                    if not ii:
                        continue
                    if ii.tag in ['ul', 'dl']:
                        self.weight += 4
                        continue
                    if ii.tag in ['div']:
                        self.weight += 1
                        continue
                # text = '' if not i.text else i.text.strip()
                # tail = '' if not i.tail else i.tail.strip()
                text = self.find_zh(etree.tostring(i[0], encoding='utf-8').decode())
                self.count_lines(i)
                c = copy.deepcopy(self.c)
                date_count = copy.deepcopy(self.date_count)
                self.c = 0
                self.date_count = 0
                self.weight += math.log(len(text) + 1) * (date_count + 1) + self.calc_node_weight_links(i)
                self.weight *= 2
                self.weight = self.weight / r
                weight = self.weight
                self.calc_links(i, r + 1)

    def get_main_links(self, url, html):
        if isinstance(html, bytes):
            encoding = cchardet.detect(html)['encoding']
            if encoding is None:
                return None
            html = html.decode(encoding, 'ignore')
        try:
            doc = lxml.html.fromstring(html)
            doc.make_links_absolute(base_url=url)
        except:
            traceback.print_exc()
            return None
        body = doc.xpath('//body')
        if not body:
            return None
        candidates = []

        def add(self, node):
            for i in node:
                if i is not None:
                    self.calc_links(i)
                    weight = copy.deepcopy(self.weight)
                    self.weight = 0
                    if weight > 0 and i.tag in ['ul', 'div', 'ol', 'dl', 'td', 'tr', 'tbody', 'table']:
                        candidates.append((i, math.log(weight, 10)))
                    add(self, i)

        add(self, body[0])
        if not candidates:
            return None
        candidates.sort(key=lambda a: a[1], reverse=True)
        good = []
        for i in candidates:
            if math.fabs(i[1] - candidates[0][1]) < 5 * 10e-3:
                good.append(copy.deepcopy(i[0]))

        # if good.tag not in ['ul', 'div']:
        #     for i in range(8):
        #         if good.tag in ['div', 'ul']:
        #             break
        #         if not good:
        #             continue
        #         good = good.getparent()
        #         if not good:
        #             break
        #         if good.tag in ['div', 'ul']:
        #             break

        # for i in candidates:
        #     print(etree.tostring(i[0], encoding='utf-8').decode(), i[1])

        # good = [self.clean_etree(i, url) for i in good]
        return good

    def get_main_block(self, url, html, short_title=True):
        """
        :param url:
        :param html:
        :param short_title:
        :return: return (title, etree_of_main_content_block)
        """
        if isinstance(html, bytes):
            encoding = cchardet.detect(html)['encoding']
            if encoding is None:
                return None, None
            html = html.decode(encoding, 'ignore')
        try:
            doc = lxml.html.fromstring(html)
            doc.make_links_absolute(base_url=url)
        except:
            traceback.print_exc()
            return None, None
        self.title = self.get_title(doc)
        if short_title:
            self.title = self.shorten_title(self.title)
        body = doc.xpath('//body')
        if not body:
            return self.title, None
        candidates = []
        nodes = body[0].getchildren()
        while nodes:
            node = nodes.pop(0)
            children = node.getchildren()
            tlen = 0
            for child in children:
                if isinstance(child, HtmlComment):
                    continue
                if child.tag in self.non_content_tag:
                    continue
                if child.tag == 'a':
                    continue
                if child.tag == 'textarea':
                    # FIXME: this tag is only part of content?
                    continue
                attr = '%s%s%s' % (child.get('class', ''),
                                   child.get('id', ''),
                                   child.get('style'))
                if 'display' in attr and 'none' in attr:
                    continue
                nodes.append(child)
                if child.tag == 'p':
                    weight = 3
                else:
                    weight = 1
                text = '' if not child.text else child.text.strip()
                tail = '' if not child.tail else child.tail.strip()
                tlen += (len(text) + len(tail)) * weight
            if tlen < 10:
                continue
            weight = self.calc_node_weight(node)
            candidates.append((node, tlen * weight))
        if not candidates:
            return self.title, None
        candidates.sort(key=lambda a: a[1], reverse=True)
        good = candidates[0][0]
        if good.tag in ['p', 'pre', 'code', 'blockquote']:
            for i in range(5):
                good = good.getparent()
                if good.tag == 'div':
                    break
        good = self.clean_etree(good, url)
        return self.title, good

    def clean_etree(self, tree, url=''):
        to_drop = []
        drop_left = False
        for node in tree.iterdescendants():
            if drop_left:
                to_drop.append(node)
                continue
            if isinstance(node, HtmlComment):
                to_drop.append(node)
                if self.p_content_stop.search(node.text):
                    drop_left = True
                continue
            if node.tag in self.non_content_tag:
                to_drop.append(node)
                continue
            attr = '%s %s' % (
                node.get('class', ''),
                node.get('id', '')
            )
            if self.p_clean_tree.search(attr):
                to_drop.append(node)
                continue
            aa = node.xpath('.//a')
            if aa:
                text_node = len(self.p_space.sub('', node.text_content()))
                text_aa = 0
                for a in aa:
                    alen = len(self.p_space.sub('', a.text_content()))
                    if alen > 5:
                        text_aa += alen
                if text_aa > text_node * 0.4:
                    to_drop.append(node)
        for node in to_drop:
            try:
                node.drop_tree()
            except:
                pass
        return tree

    def get_text(self, doc):
        lxml.etree.strip_elements(doc, 'script')
        lxml.etree.strip_elements(doc, 'style')
        for ch in doc.iterdescendants():
            if not isinstance(ch.tag, str):
                continue
            if ch.tag in ['div', 'h1', 'h2', 'h3', 'p', 'br', 'table', 'tr', 'dl']:
                if not ch.tail:
                    ch.tail = '\n'
                else:
                    ch.tail = '\n' + ch.tail.strip() + '\n'
            if ch.tag in ['th', 'td']:
                if not ch.text:
                    ch.text = '  '
                else:
                    ch.text += '  '
            # if ch.tail:
            #     ch.tail = ch.tail.strip()
        lines = doc.text_content().split('\n')
        content = []
        for l in lines:
            l = l.strip()
            if not l:
                continue
            content.append(l)
        return '\n'.join(content)

    def extract(self, url, html):
        """
        :param url:
        :param html:
        :return: return (title, content)
        """
        title, node = self.get_main_block(url, html)
        if node is None:
            print('\tno main block got !!!!!', url)
            return title, '', ''
        content = self.get_text(node)
        content_with_html = etree.tostring(node, encoding='utf-8').decode()
        return title, content, content_with_html

    def extract_links(self, url, html):
        """
        :param url:
        :param html:
        :return: return (title, content)
        """
        node = self.get_main_links(url, html)
        if node is None:
            print('\tno main block got !!!!!', url)
            return '', ''
        # for i in node:
        #     print(etree.tostring(i, encoding='utf-8').decode())
        content_ = [i.xpath('//a/@href') for i in node]
        content__ = [i.xpath('//a/text()') for i in node]
        content = []
        content_name = []
        for i in content_:
            content.extend(i)
        for i in content__:
            content_name.extend(i)
        if len(content) != len(content_name):
            content_name = content
        content = dict(zip(content_name, content))
        content_with_html = '\n'.join([etree.tostring(i, encoding='utf-8').decode() for i in node])
        return content, content_with_html


if __name__ == '__main__':
    import requests
    from fake_useragent import UserAgent

    ua = UserAgent()
    headers = dict()
    headers['User-Agent'] = ua.random
    m = TitleContent()
    # FIXME: How to identify the title of article not in title tag, but in h1 tag 'http://www.biodiscover.com/talk/investment.html'
    # url = 'http://www.biodiscover.com/talk/investment.html'

    url = 'http://www.eisai.com.cn/cn/news.php'
    # url = 'https://finance.sina.com.cn/stock/usstock/'
    # url = 'http://www.cd120.com/htmlnewszhaomuxinxi/index.jhtml'
    # url = 'http://www.ada.gov.cn/4117559.html'
    # url = 'http://www.hebfda.gov.cn/CL0417/'
    # url = 'http://scjg.tj.gov.cn/xwzx/gs/index.html'
    # url = 'http://www.gdda.gov.cn/publicfiles/business/htmlfiles/jsjzz/s2213/list.htm'
    html = requests.get(url=url, headers=headers)
    html.encoding = html.apparent_encoding
    html = html.text
    e = m.extract_links(url, html)
    print(e)
