import re


class Clean(object):

    @staticmethod
    def clean_blank_line(html_body):
        if html_body:
            blank_line = re.compile('\n+')
            clean_body = blank_line.sub('', html_body)
            return clean_body
        else:
            return

    @staticmethod
    def replace_char_entity(html_body):
        char_entities = {'nbsp': ' ', '160': ' ',
                         'lt': '<', '60': '<',
                         'gt': '>', '62': '>',
                         'amp': '&', '38': '&',
                         'quot': '"', '34': '"', }

        re_char_entity = re.compile(r'&#?(?P<name>\w+);')
        sz = re_char_entity.search(html_body)
        while sz:
            entity = sz.group()  # entity全称，如&gt;
            key = sz.group('name')  # 去除&;后entity,如&gt;为gt
            try:
                html_body = re_char_entity.sub(char_entities[key], html_body, 1)
                sz = re_char_entity.search(html_body)
            except KeyError:
                # 以空串代替
                html_body = re_char_entity.sub('', html_body, 1)
                sz = re_char_entity.search(html_body)
        return html_body

    @staticmethod
    def clean_html_tag(html_body):
        if html_body:
            re_cdata = re.compile('//<!\[CDATA\[[^>]*//\]\]>', re.I)  # 匹配CDATA
            re_script = re.compile('<\s*script[^>]*>[^<]*<\s*/\s*script\s*>', re.I)  # Script
            re_style = re.compile('<\s*style[^>]*>[^<]*<\s*/\s*style\s*>', re.I)  # style
            re_br = re.compile('<br\s*?/?>')  # 处理换行
            re_h = re.compile('</?\w+[^>]*>')  # HTML标签
            re_comment = re.compile('<!--[^>]*-->')  # HTML注释
            blank_line = re.compile('\n+')
            blank_line2 = re.compile('\r+')
            blank_line3 = re.compile('\t+')
            s = re_cdata.sub('', html_body)  # 去掉CDATA
            s = re_script.sub('', s)  # 去掉SCRIPT
            s = re_style.sub('', s)  # 去掉style
            s = re_br.sub('\n', s)  # 将br转换为换行
            s = re_h.sub('', s)  # 去掉HTML 标签
            s = re_comment.sub('', s)  # 去掉HTML注释
            # 去掉所有的空行、空格
            s = blank_line.sub('', s)
            s = blank_line2.sub('', s)
            s = blank_line3.sub('', s)
            s = Clean.replace_char_entity(s)  # 替换实体
            return s
        else:
            return

    @staticmethod
    def remove_charactor_in_table(chr, content_html):
        """
            chr: 需要去除的字符
            作用: 去除table标签中的\n. remove_charactor_in_table("\n", content_html)
        """
        all_tables = re.findall(r'(<table.*?>.*?</\s?table>)', content_html, re.S)
        remove_chr = [table.strip('\n') for table in all_tables]

        for old_table, new_table in zip(all_tables, remove_chr):
            content_html = content_html.replace(old_table, new_table)
        return content_html

    @staticmethod
    def remove_useless_tag(tag, content_html):
        """
            tag: 需要去除的空tag
            作用: 如去除<p></p>, <p><br></p>
        """
        rc = re.compile("<%s[^>]*>\s*<br\s?/?>\s*</\s?%s>" % (tag, tag))
        remove_br_tag = re.sub(rc, r'', content_html)
        rc = re.compile("<%s[^>]*>\s*</\s?%s>" % (tag, tag))
        remove_blank_tag = re.sub(rc, r'', remove_br_tag)
        return remove_blank_tag

    @staticmethod
    def remove_charactor_in_p(chr, content_html):
        """
            chr: 需要去除的字符
            作用: 去除p标签之后的<br>. remove_charactor_in_p("<br>", content_html)
        """
        all_ps = re.findall(r'(<p[^>]*>\s*<br>.*?<br>\s*</\s?p>)', content_html, re.S)
        remove_chr = [p.replace(chr, "") for p in all_ps]

        for old_p, new_p in zip(all_ps, remove_chr):
            content_html = content_html.replace(old_p, new_p)
        return content_html

    @staticmethod
    def remove_continuous_tag(tag, content_html):
        """
            tag: 需要去除的标签
            作用: 连续的tag只保留一个,如去掉br标签：remove_continuous_tag("br", content_html)
        """
        rc = re.compile("(<%s\s?/?>\n){2,}" % (tag,))
        content_html = re.sub(rc, r'', content_html)
        return content_html

    @staticmethod
    def delete_style_attribute(tag, content_html):
        """
            作用: 去除tag中的style属性
            tag: 需要去除的tag
        """
        all_tags = re.findall(r"<%s.*?/?\s?>" % tag, content_html, re.S)
        remove_chr = [re.sub(r'style=".*?" | style=\'.*?\'', r"", tag) for tag in all_tags]

        for old_table, new_table in zip(all_tags, remove_chr):
            content_html = content_html.replace(old_table, new_table)
        return content_html

    @staticmethod
    def delete_all_style_attributes(content_html):
        """
            作用: 去除table中所有tag的style属性
        """
        all_tables = re.findall(r'(<table.*?>.*?</\s?table>)', content_html, re.S)
        remove_chr = [re.sub(r'style=".*?"', r"", table) for table in all_tables]

        for old_table, new_table in zip(all_tables, remove_chr):
            content_html = content_html.replace(old_table, new_table)
        return content_html
