# coding=utf-8

from docx import Document
from docx.shared import Pt
from io import BytesIO
from docx.shared import Inches


def write_docx(fname):
    # 打开文档
    document = Document()

    # 加入不同等级的标题
    document.add_heading('MS WORD写入测试', 0)
    document.add_heading('一级标题', 1)
    document.add_heading('二级标题', 2)

    # 添加文本
    paragraph = document.add_paragraph('我们在做文本测试！')

    # 设置字号
    run = paragraph.add_run('设置字号、')
    run.font.size = Pt(24)

    # 设置字体
    run = paragraph.add_run('Font')
    run.font.name = 'Consolas'

    # 设置斜体
    run = paragraph.add_run('斜体、')
    run.italic = True

    # 设置粗体
    run = paragraph.add_run('粗体').bold = True

    # 增加引用
    document.add_paragraph('Intense quote', style='IntenseQuote')

    # 增加无序列表
    document.add_paragraph(
        '无序列表元素1', style='ListBullet'
    )

    document.add_paragraph(
        '无序列表元素2', style='ListBullet'
    )

    # 增加有序列表
    document.add_paragraph(
        '有序列表元素1', style='ListNumber'
    )

    document.add_paragraph(
        '有序列表元素2', style='ListNumber'
    )

    # 增加图像
    # document.add_picture('image.bmp', width=Inches(1.25))

    # 增加表格
    table = document.add_table(rows=1, cols=3)
    hdr_cells = table.rows[0].cells
    hdr_cells[0].text = 'Name'
    hdr_cells[1].text = 'Id'
    hdr_cells[2].text = 'Desc'

    # 再增加3行表格元素
    for i in range(3):
        row_cells = table.add_row().cells
        row_cells[0].text = 'test' + str(i)
        row_cells[1].text = str(i)
        row_cells[2].text = 'desc' + str(i)

    # 增加分页
    document.add_page_break()

    # 保存文件
    document.save(fname)


def read_docx(fname):
    # 打开文档
    document = Document(fname)

    # 读取每段资料
    p = [paragraph.text for paragraph in document.paragraphs]

    del document

    # 输出结果
    result = '\n'.join(p)
    return result


from pdfminer.pdfparser import PDFParser, PDFDocument
from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
from pdfminer.converter import PDFPageAggregator
from pdfminer.layout import LTTextBoxHorizontal, LAParams
from pdfminer.pdfinterp import PDFTextExtractionNotAllowed

'''
 解析pdf 文本，保存到txt文件中
'''


def convert_pdf_to_txt(path, txt='test'):
    fp = open(path, 'rb')  # 以二进制读模式打开
    # 用文件对象来创建一个pdf文档分析器
    praser = PDFParser(fp)
    # 创建一个PDF文档
    doc = PDFDocument()
    # 连接分析器 与文档对象
    praser.set_document(doc)
    doc.set_parser(praser)

    # 提供初始化密码
    # 如果没有密码 就创建一个空的字符串
    doc.initialize()

    # 检测文档是否提供txt转换，不提供就忽略
    if not doc.is_extractable:
        raise PDFTextExtractionNotAllowed
    else:
        # 创建PDf 资源管理器 来管理共享资源
        rsrcmgr = PDFResourceManager()
        # 创建一个PDF设备对象
        laparams = LAParams()
        device = PDFPageAggregator(rsrcmgr, laparams=laparams)
        # 创建一个PDF解释器对象
        interpreter = PDFPageInterpreter(rsrcmgr, device)

        # 循环遍历列表，每次处理一个page的内容
        for page in doc.get_pages():  # doc.get_pages() 获取page列表
            interpreter.process_page(page)
            # 接受该页面的LTPage对象
            layout = device.get_result()
            results = ''
            # 这里layout是一个LTPage对象 里面存放着 这个page解析出的各种对象 一般包括LTTextBox, LTFigure, LTImage, LTTextBoxHorizontal 等等 想要获取文本就获得对象的text属性，
            for x in layout:
                if (isinstance(x, LTTextBoxHorizontal)):
                    # with open(txt, 'a') as f:
                    #     results = x.get_text()
                    #     print(results)
                    #     f.write(results + '\n')
                    results += x.get_text()
            return results


if __name__ == '__main__':
    # write_doc('test1.docx')
    # read_doc('test1.docx')
    convert_pdf_to_txt('test.pdf', 'test.txt')
