import os
import sys
import logging
import traceback

from raven import Client
from raven.handlers.logging import SentryHandler
from raven.utils.compat import iteritems

env = 'dev'
if os.environ.get('CURRENT_ENV'):
    env = os.environ.get('CURRENT_ENV')

RAVEN_CLIENT = os.environ.get('RAVEN_CLIENT')

RESERVED = frozenset((
    'stack', 'name', 'module', 'funcName', 'args', 'msg', 'levelno',
    'exc_text', 'exc_info', 'data', 'created', 'levelname', 'msecs',
    'relativeCreated', 'tags', 'message',
))

CONTEXTUAL = frozenset((
    'user', 'culprit', 'server_name', 'fingerprint'
))


def extract_extra(record, reserved=RESERVED, contextual=CONTEXTUAL):
    data = {}

    extra = getattr(record, 'data', None)
    if not isinstance(extra, dict):
        if extra:
            extra = {'data': extra}
        else:
            extra = {}

    for k, v in iteritems(vars(record)):
        if k in reserved:
            continue
        if k.startswith('_'):
            continue
        if '.' not in k and k not in contextual:
            extra[k] = v
        else:
            data[k] = v

    return data, extra


class KafkaHandler(logging.Handler):
    def __init__(self, client, topic):
        super().__init__()
        self.producer = client.producer
        self.topic = topic

    def emit(self, record):
        data, extra = extract_extra(record)
        """Inserting new logging record to mongo database."""
        if self.producer is not None:
            try:
                self.producer.send(self.topic, extra)
            except Exception as E:
                print("Top level Sentry exception caught - failed "
                      "creating log record", file=sys.stderr)
                print(str(record.msg), file=sys.stderr)
                print(str(traceback.format_exc()), file=sys.stderr)


class Logger(object):
    def __init__(self, name='spider-system', mode=env):
        self.name = name
        self.logger = logging.getLogger(self.name)
        self.logger.setLevel(logging.INFO)

        if not self.logger.handlers:
            handler = logging.StreamHandler()

            syslog_formatter = logging.Formatter(
                self.name + ' : %(levelname)s : %(filename)s : %(lineno)d : %(funcName)s() : %(message)s : %(asctime)s')
            handler.setFormatter(syslog_formatter)

            self.logger.addHandler(handler)

            if mode != 'dev' and RAVEN_CLIENT:
                raven_client = Client(RAVEN_CLIENT, auto_log_stacks=True)

                handler = SentryHandler(raven_client)

                handler.setLevel(logging.ERROR)

                handler.setFormatter(syslog_formatter)

                self.logger.addHandler(handler)


logger = Logger().logger

logger.info('当前环境 %s' % env)

# from framework.connect import Kafka
# from framework.settings import config

# kafka_client = Kafka(config.kafka_spider['broker'])
#
# handler = KafkaHandler(kafka_client, config.kafka_spider['topic_log'])
#
# logger.addHandler(handler)

if __name__ == '__main__':
    logger.error('2189380', extra={
        'stack': 'jwehf',
    })
