import copy
import time
import random
import traceback
import requests
import urllib3
import ostrich_json as json
from .metrics import *
from .toolkit import Toolkit
from .extractor import NewExtractor
from .format import Format
from .filter import DuplicateFilter
from .clean import Clean
from .check import Check
from .crawler import requests_backend
from .crawler import JSRenderError, NoResponseError, MethodError

DEFAULT_REQUEST_HEADERS = {
    'user-agent': Toolkit.user_agent(),
    'accept': "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/wxpic,image/sharpp,*/*;q=0.8",
    'accept-encoding': "gzip, deflate",
    'accept-language': "zh-CN,en-US;q=0.8",
}


class Schedule(object):
    def __init__(self, rc, r_input, r_output, r_log, r_proxy_pool_0, r_proxy_pool_1, logger, filter_key,
                 title_filter_key=None, k_send=None, k_trace_topic=None, extract_server=None, render_url=None,
                 process_location='list_spider',selenium_url=None):
        self.name = 'default'
        self.rc = rc
        self.r_input = r_input
        self.r_output = r_output
        self.r_log = r_log
        self.r_proxy_pool_0 = r_proxy_pool_0
        self.r_proxy_pool_1 = r_proxy_pool_1
        self.logger = logger
        self.filter = DuplicateFilter()
        self.filter_key = filter_key
        self.title_filter_key = title_filter_key
        self.extract_server = extract_server
        self.render_url = render_url
        self.selenium_url = selenium_url
        self.extract = NewExtractor()
        self.k_send = k_send
        self.k_trace_topic = k_trace_topic
        self.data = dict()
        # self.line_separate = ' /-----Traceback换行分割线-----/ '
        self.line_separate = '\n'
        self.process_location = process_location

    def try_func(self, argv=1, proxy_level=0):
        try:
            self.process_request(is_proxy=argv, proxy_level=proxy_level)
        except NoResponseError as E:
            METRICS_HTTP_FAIL_COUNTER_NoResponseError.inc()
            self.process_exceptions('返回空response错误', data=self.data,
                                    record='url:%s proxy:%s is_proxy:%s error:%s' % (
                                        self.data.get('url'), self.data.get('proxy'), argv,
                                        str(traceback.format_exc()).replace('\n', self.line_separate)), is_proxy=argv)
        except MethodError as E:
            METRICS_HTTP_FAIL_COUNTER_MethodError.inc()
            self.process_exceptions('没有指定请求方式，GET或POST', data=self.data,
                                    record='url:%s proxy:%s is_proxy:%s error:%s' % (
                                        self.data.get('url'), self.data.get('proxy'), argv,
                                        str(traceback.format_exc()).replace('\n', self.line_separate)), is_proxy=argv)
        except JSRenderError as E:
            METRICS_HTTP_FAIL_COUNTER_JSRenderError.inc()
            self.process_exceptions('JS渲染服务错误', data=self.data,
                                    record='url:%s proxy:%s is_proxy:%s error:%s' % (
                                        self.data.get('url'), self.data.get('proxy'), argv,
                                        str(traceback.format_exc()).replace('\n', self.line_separate)), is_proxy=argv)
        except requests.exceptions.SSLError as E:
            METRICS_HTTP_FAIL_COUNTER_SSLError.inc()
            self.process_exceptions('SSL错误', data=self.data, record='url:%s proxy:%s is_proxy:%s error:%s' % (
                self.data.get('url'), self.data.get('proxy'), argv,
                str(traceback.format_exc()).replace('\n', self.line_separate)), is_proxy=argv)
        except requests.exceptions.ProxyError as E:
            METRICS_HTTP_FAIL_COUNTER_ProxyError.inc()
            self.process_exceptions('IP代理错误', data=self.data,
                                    record='url:%s proxy:%s is_proxy:%s error:%s' % (
                                        self.data.get('url'), self.data.get('proxy'), argv,
                                        str(traceback.format_exc()).replace('\n', self.line_separate)), is_proxy=argv)
        except requests.exceptions.ConnectTimeout as E:
            METRICS_HTTP_FAIL_COUNTER_ConnectTimeout.inc()
            self.process_exceptions('连接超时错误', data=self.data,
                                    record='url:%s proxy:%s is_proxy:%s error:%s' % (
                                        self.data.get('url'), self.data.get('proxy'), argv,
                                        str(traceback.format_exc()).replace('\n', self.line_separate)), is_proxy=argv)
        except requests.exceptions.ReadTimeout as E:
            METRICS_HTTP_FAIL_COUNTER_ReadTimeout.inc()
            self.process_exceptions('连接读取超时错误', data=self.data,
                                    record='url:%s proxy:%s is_proxy:%s error:%s' % (
                                        self.data.get('url'), self.data.get('proxy'), argv,
                                        str(traceback.format_exc()).replace('\n', self.line_separate)), is_proxy=argv)
        except requests.exceptions.ConnectionError as E:
            METRICS_HTTP_FAIL_COUNTER_ConnectionError.inc()
            self.process_exceptions('连接错误', data=self.data, record='url:%s proxy:%s is_proxy:%s error:%s' % (
                self.data.get('url'), self.data.get('proxy'), argv,
                str(traceback.format_exc()).replace('\n', self.line_separate)), is_proxy=argv)
        except urllib3.exceptions.ReadTimeoutError as E:
            METRICS_HTTP_FAIL_COUNTER_ReadTimeoutError.inc()
            self.process_exceptions('连接读取超时错误', data=self.data,
                                    record='url:%s proxy:%s is_proxy:%s error:%s' % (
                                        self.data.get('url'), self.data.get('proxy'), argv,
                                        str(traceback.format_exc()).replace('\n', self.line_separate)), is_proxy=argv)
        except urllib3.exceptions.MaxRetryError as E:
            METRICS_HTTP_FAIL_COUNTER_MaxRetryError.inc()
            self.process_exceptions('最大重试次数错误', data=self.data,
                                    record=str(traceback.format_exc()).replace('\n', self.line_separate), is_proxy=argv)
        except urllib3.exceptions.NewConnectionError as E:
            METRICS_HTTP_FAIL_COUNTER_NewConnectionError.inc()
            self.process_exceptions('新建连接错误', data=self.data,
                                    record=str(traceback.format_exc()).replace('\n', self.line_separate), is_proxy=argv)
        except urllib3.exceptions.ConnectTimeoutError as E:
            METRICS_HTTP_FAIL_COUNTER_ConnectTimeoutError.inc()
            self.process_exceptions('连接超时错误', data=self.data,
                                    record='url:%s proxy:%s is_proxy:%s error:%s' % (
                                        self.data.get('url'), self.data.get('proxy'), argv,
                                        str(traceback.format_exc()).replace('\n', self.line_separate)), is_proxy=argv)
        except urllib3.exceptions.ProtocolError as E:
            METRICS_HTTP_FAIL_COUNTER_ProtocolError.inc()
            self.process_exceptions('协议错误', data=self.data,
                                    record=str(traceback.format_exc()).replace('\n', self.line_separate), is_proxy=argv)
        except ConnectionResetError as E:
            METRICS_HTTP_FAIL_COUNTER_ConnectionResetError.inc()
            self.process_exceptions('连接重置错误', data=self.data,
                                    record=str(traceback.format_exc()).replace('\n', self.line_separate), is_proxy=argv)
        except Exception as E:
            METRICS_HTTP_FAIL_COUNTER_UnknownError.inc()
            self.process_exceptions('未定义错误类型', data=self.data,
                                    record=str(traceback.format_exc()).replace('\n', self.line_separate), is_proxy=argv)

    def process_input(self):
        self.data['process_location'] = self.process_location
        self.data = self.rc.json_pop(self.r_input)
        self.data['retry_count'] = 0
        self.data['next_layer_initial'] = ['proxy', 'error_process', 'error_record', 'current_layer_links',
                                           'current_layer_item', 'current_layer_not_null', 'current_layer_aligning']
        self.data['current_layer_links'] = []
        self.data['current_layer_item'] = []
        self.data['current_layer_not_null'] = []
        self.data['current_layer_aligning'] = []
        if not self.data.get('request_data'):
            self.data['request_data'] = dict()
        if self.data.get('timeout') and isinstance(self.data.get('timeout'), int):
            self.data['timeout'] = int(self.data.get('timeout'))
        else:
            self.data['timeout'] = 3
            self.process_exceptions(name='无效的默认timeout', retry_limit=-1, data=self.data,
                                    record='无效的默认timeout，请重新设置 %s timeout: %s' % (
                                        self.data['url'], self.data.get('timeout')))
            self.logger.debug('pop list spider url success %s' % self.data['url'])

    def process_request(self, is_proxy=1, proxy_level=0):
        METRICS_HTTP_SUCCESS_COUNTER.inc()  # point

        # if self.data.get('is_js'):
        #     proxy_level = 1

        proxy = self.proxy(is_proxy=is_proxy, format_for_requests=1, p_level=proxy_level)

        self.data['proxy_level'] = proxy_level

        self.data['proxy'] = proxy.get('http')

        self.data['is_proxy'] = is_proxy

        if not self.data.get('timeout'):
            self.data['timeout'] = 3

        response = requests_backend(url=self.data['url'], headers=DEFAULT_REQUEST_HEADERS, proxies=proxy,
                                    is_proxy=is_proxy, is_render=self.data.get('is_js'),
                                    method=self.data['request_method'],
                                    parameters_data=self.data.get('request_data'),
                                    timeout=self.data.get('timeout'),
                                    render_url=self.render_url,
                                    selenium_url=self.selenium_url)

        if response.status_code == 200:
            METRICS_HTTP_SUCCESS_200_COUNTER.inc()  # point
            self.logger.info('已爬取 %s %s' % (str(response), self.data['url']))

            self.data['html'] = response.text

            if self.extract_all(data=self.data):
                self.next_links(data=self.data)

                self.filter_confirm(data=self.data, choice=1)  # 局部去重
                self.record(self.data)
        else:
            # if response.status_code in [504, 400, 404]:
            #     retry_limit = -1
            # else:
            retry_limit = 3
            self.process_exceptions('返回值非200错误', data=self.data, retry_limit=retry_limit,
                                    record='HTTP状态码是:%s  网址是:%s' % (
                                        str(response.status_code), self.data.get('url')))

    def next_links(self, data):
        data = copy.deepcopy(data)
        if data.get('current_layer_links'):
            for key in data.get('current_layer_links'):
                self.recursive(data=data, next_link_name=key)
        else:
            self.end_point(data)

    def recursive(self, data, next_link_name):
        parent_data = self.data
        parent_data['process_location'] = 'list-spider'
        initial_columns = parent_data['next_layer_initial']
        parent_rule = parent_data.get('rule')

        current_rule_content = parent_rule.get(next_link_name)
        current_rule_trace = current_rule_content.get('trace_links')

        data['process_location'] = 'list-spider'
        child_links = parent_data.get(next_link_name)
        child_rule = current_rule_content.get('rule')
        child_method = current_rule_content.get('method')
        child_method_params_name = current_rule_content.get('method_param_column_name')

        layer_item = parent_data['current_layer_item']
        wait_for_aligning = parent_data['current_layer_aligning']

        # list_pub_time = parent_data.get('list_pub_time')
        # list_title = parent_data.get('list_title')

        for initial_column in initial_columns:
            try:
                data.pop(initial_column)
            except Exception:
                pass
        if current_rule_trace:
            data['rule'] = child_rule
            data['request_method'] = child_method
            if child_method_params_name:
                for child_param_name in child_method_params_name:
                    data['request_data'][child_param_name] = parent_data[child_param_name]
            for index in range(0, len(child_links)):
                for item_name in layer_item:
                    data[item_name] = self.aligning(parent_data=parent_data, key=item_name, index=index,
                                                    columns=wait_for_aligning)
                url = child_links[index]
                data['url'] = url
                data['url_md5'] = Toolkit.md5(url)
                data['referrer_md5'] = parent_data['url_md5']
                data['url_type'] = [next_link_name]
                data['dont_filter'] = False
                data['push_dont_filter'] = False
                # if list_pub_time and isinstance(list_pub_time, list):
                #     try:
                #         data['pub_time'] = list_pub_time[parent_data['article'].index(url)]
                #     except IndexError:
                #         self.process_exceptions(name='需要规则审查', data=parent_data, record=['list_pub_time'])
                # if list_title and isinstance(list_title, list):
                #     try:
                #         data['title'] = list_title[parent_data['article'].index(url)]
                #     except IndexError:
                #         self.process_exceptions(name='需要规则审查', data=parent_data, record=['list_title'])
                if self.filter_verify_not(data=data, choice=2) and (data.get('url') != parent_data.get('url')):
                    data['url_record'].append(data.get('url'))
                    self.rc.json_push(self.r_input, data)
                    self.record(data)
                    self.logger.info('create and push recursive spider task url success %s' % data['url'])
                else:
                    self.logger.info('已过滤 article spider url %s' % data['url'])

    def end_point(self, data):

        data['process_location'] = 'article-spider'
        data['is_detail'] = 1

        if self.extract_all(data=data):
            data = self.data_normalize(data)
            if 'jobmd' in data['send_to']:
                if not self.filter_verify_not(data=data, choice=3):
                    data['send_to'] = data['send_to'].remove('jobmd')
                    self.logger.debug(data['url'] + 'jobmd url 被按照标题去重过滤')  # 临时去重调用
                    self.process_exceptions(name='jobmd按标题去重过滤', record='', data=data, retry_limit=-1)  # 跟踪日志
            self.rc.json_push(self.r_output, data)
            self.record(data)  # 跟踪日志
            self.logger.info('已成功详情页 %s %s' % (data.get('title'), data.get('url')))
            self.filter_confirm(data=data, choice=2)  # 全局去重

    def process_exceptions(self, name, data, retry_limit=3, record=None, is_proxy=1):
        # self.logger.error('%s failure : %s' % ('news spider', name,), extra=self.data)
        if int(data['retry_count']) < int(retry_limit):
            data['retry_count'] = data['retry_count'] + 1
            if data['retry_count'] >= retry_limit - 1:
                proxy_level = 1
                p_flag = random.choice([1, 2, 3, 4, 5])
                if p_flag == 1:
                    return self.try_func(argv=0, proxy_level=proxy_level)
                else:
                    pass
                return self.try_func(argv=is_proxy, proxy_level=proxy_level)
            else:
                return self.try_func(argv=is_proxy, proxy_level=0)  # 0表示本机IP
        else:
            self.logger.info(
                '%s 记录至kafka : %s %s %s sleep : %ss' % ('news spider', name, record, data.get('url'), 0))
            data['error_process'] = name
            data['error_record'] = record
            data['catch_time'] = int(time.time())
            self.rc.json_push(self.r_log, data)
            time.sleep(0)

    def extract_all(self, data):

        if data.get('rule'):
            for k, v in data.get('rule').items():
                data.update(
                    self.extract.extract(url=data.get('url'), column_name=k, xpath=v.get('xpath'),
                                         regex=v.get('regex'),
                                         html=data.get('html'), is_link=v.get('is_link'), data=data,
                                         get_all=v.get('get_all')))
                if data.get('rule').get(k).get('trace_links'):
                    data['current_layer_links'].append(k)
                else:
                    data['current_layer_item'].append(k)
                if not data.get('rule').get(k).get('is_null'):
                    data['current_layer_not_null'].append(k)
                if not data.get('rule').get(k).get('dont_aligning'):
                    data['current_layer_aligning'].append(k)

        if self.extract_server and data.get('is_detail'):
            wd = self.extract.web_collector(url=data.get('url'), html=data.get('html'),
                                            extract_server=self.extract_server)
            if not data.get('pub_time'):
                data['pub_time'] = wd.get('pub_time')
            if not data.get('title'):
                data['title'] = wd.get('title')
            if not data.get('content'):
                data['content'] = wd.get('content')
            if not data.get('content_html'):
                data['content_html'] = wd.get('content_html')
            data['current_layer_not_null'].extend(['pub_time', 'title', 'content', 'content_html'])
            data['current_layer_item'].extend(['pub_time', 'title', 'content', 'content_html'])
            data['current_layer_not_null'].extend(['pub_time', 'title', 'content', 'content_html'])

        rule_review = Check.check_blank(item=data, columns=data.get('current_layer_not_null'))
        if data.get('is_detail'):
            if data.get('title'):
                data['title'] = Clean.clean_blank_line(Clean.clean_html_tag(data.get('title')))
            try:
                data['pub_time'] = Format.time_normalization(data.get('pub_time'))
            except ValueError:
                try:
                    data['pub_time'] = Format.time_normalization(Clean.clean_html_tag(data.get('pub_time')))
                except ValueError:
                    rule_review.append('pub_time')
        rule_review = list(set(rule_review))
        self.data_normalize(data)
        if rule_review:
            self.process_exceptions(name='需要规则审查', data=data, retry_limit=-1,
                                    record='链接为 : %s 字段为 : %s' % (data.get('url'), rule_review))
            return
        else:
            return data

    @staticmethod
    def data_normalize(data):
        if not data.get('url_record'):
            data['url_record'] = []
        data['url_record'].append(data.get('url'))

        if data.get('rule'):
            data['current_layer_links'] = list(set(data['current_layer_links']))
            data['current_layer_item'] = list(set(data['current_layer_item']))
            data['current_layer_not_null'] = list(set(data['current_layer_not_null']))
            data['current_layer_aligning'] = list(set(data['current_layer_aligning']))

        if data.get('is_detail'):
            if data.get('title'):
                data['title'] = Clean.clean_blank_line(Clean.clean_html_tag(data.get('title')))
            if data.get("content_html"):
                if 'insight' in data['send_to']:
                    data["content_html"] = Clean.remove_charactor_in_table("\n", data["content_html"])
                    # 去除table标签内容中的\n
                    data["content_html"] = Clean.remove_useless_tag("p", data["content_html"])
                    # 去除无用的标签，这里产品提出要去掉<p></p>或者<p><br></p>
                    data["content_html"] = Clean.delete_style_attribute("img", data["content_html"])
                    # 去除img标签的style样式
                    data["content_html"] = Clean.delete_all_style_attributes(data["content_html"])
                    # 去除table标签的stle样式，以及里面所有的标签
                    data["content_html"] = Clean.remove_charactor_in_p("<br>", data["content_html"])
                    # 去除p标签内无用的<br>
                    data["content_html"] = Clean.remove_continuous_tag("br", data["content_html"])
                    # 去除连续的tag,只保留一个，如<br><br><br>只保留一个
                    data['content'] = Clean.clean_html_tag(data['content_html'])
                    data['pub_time'] = Clean.clean_blank_line(Clean.clean_html_tag(data.get('pub_time')))
        return data

    @staticmethod
    def aligning(parent_data, key, index, columns):
        if key in columns:
            return parent_data.get(key)[index]
        else:
            return parent_data.get(key)

    def record(self, data):
        json_data = copy.deepcopy(data)
        json_data['data_type'] = '跟踪日志'
        json_data['catch_time'] = int(time.time())
        task = json.dumps(json_data)
        if isinstance(task, str):
            task = task.encode()
        if isinstance(task, bytes):
            if self.k_send and self.k_trace_topic:
                self.k_send(self.k_trace_topic, task)
        else:
            self.logger.error(
                '跟踪日志格式错误，无法导入 : %s %s %s %s' % (
                    json_data.get('title'), json_data.get('url'), json_data.get('send_to'),
                    json_data.get('process_location')), exrta={'before_dumps': json_data, 'after_dumps': task})

    def proxy(self, is_proxy, p_level=1, format_for_requests=1):
        try:
            if is_proxy and p_level == 0:
                ip_raw = self.rc.srandmember(self.r_proxy_pool_0)
                PROXY_COUNTER.inc()  # point
                ip_json = json.loads(ip_raw.decode())
                # rc.sadd(self.r_proxy_pool_0, json.dumps(ip_json))
                ip = ip_json.get('ip') + ':' + str(ip_json.get('port'))
                expire_time = Format.str_datetime_timestamp(
                    Format.datetime_str(Format.str_datetime(ip_json.get('expire_time'))))
                if expire_time < time.time():
                    self.rc.srem(self.r_proxy_pool_0, ip_raw)
                    self.logger.debug('get proxy success %s' % ip)
                    time.sleep(1)
                    self.logger.error('temp proxy expired', extra=ip_json)
                    return self.proxy(is_proxy=is_proxy, format_for_requests=format_for_requests, p_level=p_level)
                if format_for_requests:
                    proxies = {
                        "http": "http://" + ip,
                        "https": "http://" + ip,
                    }
                    return proxies
                else:
                    return ip

            elif is_proxy and p_level == 1:
                ip_raw = self.rc.srandmember(self.r_proxy_pool_1)
                PROXY_COUNTER.inc()  # point
                ip_json = json.loads(ip_raw.decode())
                # rc.sadd(self.r_proxy_pool_1, json.dumps(ip_json))
                ip = ip_json.get('ip') + ':' + str(ip_json.get('port'))
                expire_time = Format.str_datetime_timestamp(
                    Format.datetime_str(Format.str_datetime(ip_json.get('expire_time'))))
                if expire_time < time.time():
                    self.rc.srem(self.r_proxy_pool_1, ip_raw)
                    self.logger.debug('get proxy success %s' % ip)
                    time.sleep(1)
                    self.logger.error('long term proxy expired', extra=ip_json)
                    return self.proxy(is_proxy=is_proxy, format_for_requests=format_for_requests, p_level=p_level)
                if format_for_requests:
                    proxies = {
                        "http": "http://" + ip,
                        "https": "http://" + ip,
                    }
                    return proxies
                else:
                    return ip
            else:
                if format_for_requests:
                    return dict()
                else:
                    return ''
        except Exception as E:
            time.sleep(2)
            self.logger.error('no proxy', extra={'代理用尽': str(traceback.format_exc())})
            return self.proxy(is_proxy=is_proxy, format_for_requests=format_for_requests, p_level=p_level)

    def filter_verify_not(self, data, choice=2):
        condition_global = not self.filter.verify(value=Toolkit.md5(data.get('url')), rc=self.rc,
                                                  key=self.filter_key,
                                                  dont_filter=data.get('dont_filter'))
        condition_batch = not self.filter.verify(value=Toolkit.md5(data.get('url')), rc=self.rc,
                                                 key='batch_id_%s' % data.get('batch_id'),
                                                 dont_filter=False)
        condition_all = condition_batch and condition_global

        condition_title = not self.filter.verify(value=data.get('title'), rc=self.rc, key=self.title_filter_key)

        condition = [condition_global, condition_batch, condition_all, condition_title]

        return condition[choice]

    def filter_confirm(self, data, choice=2):
        if choice == 0:
            self.filter.confirm(value=Toolkit.md5(data.get('url')), rc=self.rc, key=self.filter_key)
        if choice == 1:
            self.filter.confirm(value=Toolkit.md5(data.get('url')), rc=self.rc,
                                key='batch_id_%s' % data.get('batch_id'))
        if choice == 2:
            self.filter.confirm(value=Toolkit.md5(data.get('url')), rc=self.rc, key=self.filter_key)
            self.filter.confirm(value=Toolkit.md5(data.get('url')), rc=self.rc,
                                key='batch_id_%s' % data.get('batch_id'))
        if choice == 3:
            self.filter.confirm(value=data.get('title'), rc=self.rc, key=self.title_filter_key)

    def main(self, is_proxy=1):
        self.process_input()
        if self.filter_verify_not(data=self.data, choice=2):  # 不在去重键内，就通过
            self.try_func(is_proxy)
        else:
            self.logger.info('已过滤 article spider url %s' % self.data['url'])
