import os
import re
import time
import random
import hashlib
import datetime
import traceback
from urllib.parse import urljoin

from lxml import etree
from tld import get_tld
from tld.exceptions import TldDomainNotFound, TldBadUrl

import ostrich_json as json


class Toolkit(object):
    @staticmethod
    def md5(data):
        if data:
            m = hashlib.md5()
            m.update(str(data).encode())
            return m.hexdigest()
        else:
            return None

    @staticmethod
    def domain_it(url):
        try:
            domain = get_tld(url)
        except (AttributeError, TldDomainNotFound, TldBadUrl) as E:
            domain = ''
        return domain

    @staticmethod
    def url_safe(url, decode='utf-8', logger=None):
        if isinstance(url, bytes):
            if decode:
                url = url.decode(decode)
            else:
                url = url.decode('utf-8')
                if logger:
                    logger.warn('url is a bytes , use default utf-8')
        if isinstance(url, str):
            if 'http://' not in url and 'https://' not in url:
                url = 'http://' + url
            return url

    @staticmethod
    def to_eval(data):
        try:
            data = eval(data)
        except (NameError, SyntaxError) as E:
            pass
        return data

    @staticmethod
    def dict_to_attr(self, dic):
        for i in dic.items():
            if isinstance(i[1], str):
                exec('''self.%s = "%s"''' % (i[0], i[1]))
            elif isinstance(i[1], dict) or isinstance(i[1], list) or isinstance(i[1], set):
                exec('''self.%s = %s''' % (i[0], i[1]))

    @staticmethod
    def user_agent():
        ua0 = 'Mozilla/5.0 '
        ua1 = ['(Windows NT 5.1; WOW32) ', '(Windows NT 6.0; WOW64) ', '(Windows NT 6.1; WOW64) ',
               '(Windows NT 6.2; WOW64) ', '(Windows NT 10.0; WOW64) ', '(Windows NT 10.0; WOW64) ',
               '(Windows NT 10.0; WOW64) ', '(Windows NT 10.0; WOW64) ', '(Windows NT 10.0; WOW64) ']
        ua2 = 'AppleWebKit/'  # + aw
        ua3 = ' (KHTML, like Gecko) Version/4.0 '
        ua4 = 'Chrome/'  # + cm
        # ua5 = ' Mobile MQQBrowser/6.2 TBS/043313 '
        ua6 = ' Safari/'  # + saf
        # ua7 = 'MicroMessenger/6.5.10.1080 NetType/WIFI Language/zh_CN'
        cmv = '{a}.0.{b}.{c}'
        ua = ua0 + random.choice(
            ua1) + ua2 + str(round(random.uniform(535, 560), 1)) + ua3 + ua4 + cmv.format(
            a=random.randint(20, 55), b=random.randint(1003, 2955), c=random.randint(0, 99)) + ua6 + str(
            round(random.uniform(535, 560), 1))
        return ua

    @staticmethod
    def extract_attachment(content_html, article_url, arg):
        tree = etree.fromstring(content_html, etree.HTMLParser())

        res = []
        anodes = tree.xpath(arg)
        for node in anodes:
            anchors = node.xpath('child::*/text()|text()')
            if len(anchors) > 0:
                t = anchors[0].strip()
            else:
                t = ''
            urls = node.xpath('@href')
            if len(urls) > 0:
                url = urls[0]
            else:
                url = ''
            if t.endswith('.doc') or t.endswith('.docx') or t.endswith('.pdf') or t.endswith('.xlsx') or t.endswith(
                    '.xls') \
                    or url.endswith('doc') or url.endswith('.docx') or url.endswith('.pdf') or url.endswith(
                '.xlsx') or url.endswith('.xls'):
                url = urljoin(article_url, url)
                res.append({t: url})
        return json.dumps(res, ensure_ascii=False)

    @staticmethod
    def get_env(setting):
        for k, v in setting.items():
            temp = os.environ.get(k)
            if temp:
                setting[k] = temp
        return setting

